require('.classes-1')

----
--- Remove items from this entity.
--- 
--- @param items ItemStackIdentification The items to remove.
--- @return uint The number of items that were actually removed.
----
function LuaControl.remove_item(items) end

----
--- @class LuaControl.set_gui_arrow_parameters
--- Parameters class for the method `LuaControl.set_gui_arrow`
--- 
--- @field type string Where to point to. This field determines what other fields are mandatory. May be `"nowhere"`, `"goal"`, `"entity_info"`, `"active_window"`, `"entity"`, `"position"`, `"crafting_queue"`, or `"item_stack"`.
----

----
--- Create an arrow which points at this entity. This is used in the tutorial. For examples, see `control.lua` in the campaign missions.
--- 
--- @param args LuaControl.set_gui_arrow_parameters Table containing arguments for the method.
--- @see LuaControl.set_gui_arrow_parameters
----
function LuaControl.set_gui_arrow(args) end

----
--- Sets a personal logistic request and auto-trash slot to the given value.
--- 
--- @param slot_index uint The slot to set.
--- @param value LogisticParameters The logistic request parameters.
--- @return boolean Whether the slot was set successfully. `false` if personal logistics are not researched yet.
----
function LuaControl.set_personal_logistic_slot(slot_index, value) end

----
--- Sets a vehicle logistic request and auto-trash slot to the given value. Only used on `spider-vehicle`.
--- 
--- @param slot_index uint The slot to set.
--- @param value LogisticParameters The logistic request parameters.
--- @return boolean Whether the slot was set successfully. `false` if the vehicle does not use logistics.
----
function LuaControl.set_vehicle_logistic_slot(slot_index, value) end

----
--- Teleport the entity to a given position, possibly on another surface.
--- 
--- @param position MapPosition Where to teleport to.
--- @param surface SurfaceIdentification? Surface to teleport to. If not given, will teleport to the entity's current surface. Only players, cars, and spidertrons can be teleported cross-surface.
--- @param raise_teleported boolean? If true, [defines.events.script_raised_teleported](defines.events.script_raised_teleported) will be fired on successful entity teleportation.
--- @return boolean `true` if the entity was successfully teleported.
----
function LuaControl.teleport(position, surface, raise_teleported) end

----
--- Select an entity, as if by hovering the mouse above it.
--- 
--- @param position MapPosition Position of the entity to select.
----
function LuaControl.update_selected_entity(position) end


----
--- @class LuaControlBehavior
--- The control behavior for an entity. Inserters have logistic network and circuit network behavior logic, lamps have circuit logic and so on. This is an abstract base class that concrete control behaviors inherit.
--- 
--- @field type defines.control_behavior.type The concrete type of this control behavior.
--- @field entity LuaEntity The entity this control behavior belongs to.
----
LuaControlBehavior = { }

----
--- 
--- @param wire defines.wire_type Wire color of the network connected to this entity.
--- @param circuit_connector defines.circuit_connector_id? The connector to get circuit network for. Must be specified for entities with more than one circuit network connector.
--- @return LuaCircuitNetwork The circuit network or nil.
----
function LuaControlBehavior.get_circuit_network(wire, circuit_connector) end


----
--- @class LuaCustomChartTag
--- A custom tag that shows on the map view.
--- 
--- @field icon SignalID This tag's icon, if it has one. Writing `nil` removes it.
--- @field last_user LuaPlayer? The player who last edited this tag.
--- @field position MapPosition The position of this tag.
--- @field text string 
--- @field tag_number uint The unique ID for this tag on this force.
--- @field force LuaForce The force this tag belongs to.
--- @field surface LuaSurface The surface this tag belongs to.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaCustomChartTag = { }

----
--- Destroys this tag.
--- 
----
function LuaCustomChartTag.destroy() end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaCustomChartTag.help() end


----
--- @class LuaCustomInputPrototype
--- Prototype of a custom input.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field key_sequence string The default key sequence for this custom input.
--- @field alternative_key_sequence string? The default alternative key sequence for this custom input, if any
--- @field linked_game_control string? The linked game control name, if any.
--- @field consuming string The consuming type: `"none"` or `"game-only"`.
--- @field action string The action that happens when this custom input is triggered.
--- @field enabled boolean Whether this custom input is enabled. Disabled custom inputs exist but are not used by the game.
--- @field enabled_while_spectating boolean Whether this custom input is enabled while using the spectator controller.
--- @field enabled_while_in_cutscene boolean Whether this custom input is enabled while using the cutscene controller.
--- @field include_selected_prototype boolean Whether this custom input will include the selected prototype (if any) when triggered.
--- @field item_to_spawn LuaItemPrototype? The item that gets spawned when this custom input is fired, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaCustomInputPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaCustomInputPrototype.help() end


----
--- @class LuaCustomTable
--- Lazily evaluated table. For performance reasons, we sometimes return a custom table-like type instead of a native Lua table. This custom type lazily constructs the necessary Lua wrappers of the corresponding C++ objects, therefore preventing their unnecessary construction in some cases.
--- 
--- There are some notable consequences to the usage of a custom table type rather than the native Lua table type: Iterating a custom table is only possible using the `pairs` Lua function; `ipairs` won't work. Another key difference is that custom tables cannot be serialised into a game save file -- if saving the game would require serialisation of a custom table, an error will be displayed and the game will not be saved.
--- 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaCustomTable = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaCustomTable.help() end


----
--- @class LuaDamagePrototype
--- Prototype of a damage.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field hidden boolean Whether this damage type is hidden from entity tooltips.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaDamagePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaDamagePrototype.help() end


----
--- @class LuaDeciderCombinatorControlBehavior: LuaCombinatorControlBehavior
--- Control behavior for decider combinators.
--- 
--- @field parameters DeciderCombinatorParameters This decider combinator's parameters.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaDeciderCombinatorControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaDeciderCombinatorControlBehavior.help() end


----
--- @class LuaDecorativePrototype
--- Prototype of an optimized decorative.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field collision_box BoundingBox The bounding box used for collision checking.
--- @field collision_mask CollisionMask The collision masks this decorative uses
--- @field collision_mask_with_flags CollisionMaskWithFlags 
--- @field autoplace_specification AutoplaceSpecification? Autoplace specification for this decorative prototype, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaDecorativePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaDecorativePrototype.help() end


----
--- @class LuaElectricEnergySourcePrototype
--- Prototype of an electric energy source.
--- 
--- @field buffer_capacity double 
--- @field usage_priority string 
--- @field input_flow_limit double 
--- @field output_flow_limit double 
--- @field drain double 
--- @field emissions double The emissions of this energy source in `pollution/Joule`. Multiplying it by energy consumption in `Watt` gives `pollution/second`.
--- @field render_no_network_icon boolean 
--- @field render_no_power_icon boolean 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaElectricEnergySourcePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaElectricEnergySourcePrototype.help() end


----
--- @class LuaEntity: LuaControl
--- The primary interface for interacting with entities through the Lua API. Entities are everything that exists on the map except for tiles (see [LuaTile](LuaTile)).
--- 
--- Most functions on LuaEntity also work when the entity is contained in a ghost.
--- 
--- @field name string Name of the entity prototype. E.g. "inserter" or "filter-inserter".
--- @field ghost_name string Name of the entity or tile contained in this ghost
--- @field localised_name LocalisedString Localised name of the entity.
--- @field localised_description LocalisedString 
--- @field ghost_localised_name LocalisedString Localised name of the entity or tile contained in this ghost.
--- @field ghost_localised_description LocalisedString 
--- @field type string The entity prototype type of this entity.
--- @field ghost_type string The prototype type of the entity or tile contained in this ghost.
--- @field active boolean Deactivating an entity will stop all its operations (car will stop moving, inserters will stop working, fish will stop moving etc).
--- @field destructible boolean If set to `false`, this entity can't be damaged and won't be attacked automatically. It can however still be mined.
--- @field minable boolean 
--- @field rotatable boolean When entity is not to be rotatable (inserter, transport belt etc), it can't be rotated by player using the R key.
--- @field operable boolean Player can't open gui of this entity and he can't quick insert/input stuff in to the entity when it is not operable.
--- @field health float? The current health of the entity, if any. Health is automatically clamped to be between `0` and max health (inclusive). Entities with a health of `0` can not be attacked.
--- @field direction defines.direction The current direction this entity is facing.
--- @field supports_direction boolean Whether the entity has direction. When it is false for this entity, it will always return north direction when asked for.
--- @field orientation RealOrientation The smooth orientation of this entity.
--- @field cliff_orientation CliffOrientation The orientation of this cliff.
--- @field relative_turret_orientation RealOrientation? The relative orientation of the vehicle turret, artillery turret, artillery wagon. `nil` if this entity isn't a vehicle with a vehicle turret or artillery turret/wagon.
--- @field torso_orientation RealOrientation The torso orientation of this spider vehicle.
--- @field amount uint Count of resource units contained.
--- @field initial_amount uint? Count of initial resource units contained. `nil` if this is not an infinite resource.
--- @field effectivity_modifier float Multiplies the acceleration the vehicle can create for one unit of energy. Defaults to `1`.
--- @field consumption_modifier float Multiplies the energy consumption.
--- @field friction_modifier float Multiplies the car friction rate.
--- @field driver_is_gunner boolean? Whether the driver of this car or spidertron is the gunner. If `false`, the passenger is the gunner. `nil` if this is neither a car or a spidertron.
--- @field vehicle_automatic_targeting_parameters VehicleAutomaticTargetingParameters Read when this spidertron auto-targets enemies
--- @field speed float? The current speed if this is a car, rolling stock, projectile or spidertron, or the maximum speed if this is a unit. The speed is in tiles per tick. `nil` if this is not a car, rolling stock, unit, projectile or spidertron.  Only the speed of units, cars, and projectiles are writable.
--- @field effective_speed float? The current speed of this unit in tiles per tick, taking into account any walking speed modifier given by the tile the unit is standing on. `nil` if this is not a unit.
--- @field stack LuaItemStack 
--- @field prototype LuaEntityPrototype The entity prototype of this entity.
--- @field ghost_prototype LuaTilePrototype|LuaEntityPrototype The prototype of the entity or tile contained in this ghost.
--- @field drop_position MapPosition Position where the entity puts its stuff.
--- @field pickup_position MapPosition Where the inserter will pick up items from.
--- @field drop_target LuaEntity? The entity this entity is putting its items to. If there are multiple possible entities at the drop-off point, writing to this attribute allows a mod to choose which one to drop off items to. The entity needs to collide with the tile box under the drop-off position. `nil` if there is no entity to put items to, or if this is not an entity that puts items somewhere.
--- @field pickup_target LuaEntity? The entity this inserter will attempt to pick up items from. If there are multiple possible entities at the pick-up point, writing to this attribute allows a mod to choose which one to pick up items from. The entity needs to collide with the tile box under the pick-up position. `nil` if there is no entity to pull items from.
--- @field selected_gun_index uint? Index of the currently selected weapon slot of this character, car, or spidertron. `nil` if this entity doesn't have guns.
--- @field energy double Energy stored in the entity (heat in furnace, energy stored in electrical devices etc.). always 0 for entities that don't have the concept of energy stored inside.
--- @field temperature double? The temperature of this entity's heat energy source. `nil` if this entity does not use a heat energy source.
--- @field previous_recipe LuaRecipe? The previous recipe this furnace was using, if any.
--- @field held_stack LuaItemStack The item stack currently held in an inserter's hand.
--- @field held_stack_position MapPosition Current position of the inserter's "hand".
--- @field train LuaTrain? The train this rolling stock belongs to, if any. `nil` if this is not a rolling stock.
--- @field neighbours LuaEntity[][]|LuaEntity|table<string,LuaEntity[]> A list of neighbours for certain types of entities. Applies to electric poles, power switches, underground belts, walls, gates, reactors, cliffs, and pipe-connectable entities.  - When called on an electric pole, this is a dictionary of all connections, indexed by the strings `"copper"`, `"red"`, and `"green"`. - When called on a pipe-connectable entity, this is an array of entity arrays of all entities a given fluidbox is connected to. - When called on an underground transport belt, this is the other end of the underground belt connection, or `nil` if none. - When called on a wall-connectable entity or reactor, this is a dictionary of all connections indexed by the connection direction "north", "south", "east", and "west". - When called on a cliff entity, this is a dictionary of all connections indexed by the connection direction "north", "south", "east", and "west".
--- @field belt_neighbours table<string,LuaEntity[]> The belt connectable neighbours of this belt connectable entity. Only entities that input to or are outputs of this entity. Does not contain the other end of an underground belt, see [LuaEntity::neighbours](LuaEntity::neighbours) for that. This is a dictionary with `"inputs"`, `"outputs"` entries that are arrays of transport belt connectable entities, or empty tables if no entities.
--- @field fluidbox LuaFluidBox Fluidboxes of this entity.
--- @field backer_name string? The backer name assigned to this entity. Entities that support backer names are labs, locomotives, radars, roboports, and train stops. `nil` if this entity doesn't support backer names.
--- @field entity_label string? The label on this spider-vehicle entity, if any. `nil` if this is not a spider-vehicle.
--- @field time_to_live uint The ticks left before a ghost, combat robot, highlight box or smoke with trigger is destroyed.  - for ghosts set to uint32 max (4,294,967,295) to never expire. - for ghosts Cannot be set higher than [LuaForce::ghost_time_to_live](LuaForce::ghost_time_to_live) of the entity's force.
--- @field color Color? The color of this character, rolling stock, train stop, car, spider-vehicle, flying text, corpse or simple-entity-with-owner. `nil` if this entity doesn't use custom colors.
--- @field text LocalisedString The text of this flying-text entity.
--- @field signal_state defines.signal_state The state of this rail signal.
--- @field chain_signal_state defines.chain_signal_state The state of this chain signal.
--- @field to_be_looted boolean Will this entity be picked up automatically when the player walks over it?
--- @field crafting_speed double The current crafting speed, including speed bonuses from modules and beacons.
--- @field crafting_progress float The current crafting progress, as a number in range [0, 1].
--- @field bonus_progress double The current productivity bonus progress, as a number in range [0, 1].
--- @field productivity_bonus double The productivity bonus of this entity.
--- @field pollution_bonus double The pollution bonus of this entity.
--- @field speed_bonus double The speed bonus of this entity.
--- @field consumption_bonus double The consumption bonus of this entity.
--- @field belt_to_ground_type string Whether this underground belt goes into or out of the ground.
--- @field loader_type string Whether this loader gets items from or puts item into a container.
--- @field rocket_parts uint Number of rocket parts in the silo.
--- @field logistic_network LuaLogisticNetwork The logistic network this entity is a part of, or `nil` if this entity is not a part of any logistic network.
--- @field logistic_cell LuaLogisticCell The logistic cell this entity is a part of. Will be `nil` if this entity is not a part of any logistic cell.
--- @field item_requests table<string,uint> Items this ghost will request when revived or items this item request proxy is requesting. Result is a dictionary mapping each item prototype name to the required count.
--- @field player LuaPlayer? The player connected to this character, if any.
--- @field unit_group LuaUnitGroup? The unit group this unit is a member of, if any.
--- @field damage_dealt double The damage dealt by this turret, artillery turret, or artillery wagon.
--- @field kills uint The number of units killed by this turret, artillery turret, or artillery wagon.
--- @field last_user PlayerIdentification|LuaPlayer? The last player that changed any setting on this entity. This includes building the entity, changing its color, or configuring its circuit network. `nil` if the last user is not part of the save anymore.  Reading this property will return a [LuaPlayer](LuaPlayer), while [PlayerIdentification](PlayerIdentification) can be used when writing.
--- @field electric_buffer_size double? The buffer size for the electric energy source. `nil` if the entity doesn't have an electric energy source.
--- @field electric_input_flow_limit double? The input flow limit for the electric energy source. `nil` if the entity doesn't have an electric energy source.
--- @field electric_output_flow_limit double? The output flow limit for the electric energy source. `nil` if the entity doesn't have an electric energy source.
--- @field electric_drain double? The electric drain for the electric energy source. `nil` if the entity doesn't have an electric energy source.
--- @field electric_emissions double? The emissions for the electric energy source. `nil` if the entity doesn't have an electric energy source.
--- @field unit_number uint? A unique number identifying this entity for the lifetime of the save. These are allocated sequentially, and not re-used (until overflow).  Only entities inheriting from [EntityWithOwner](https://wiki.factorio.com/Prototype/EntityWithOwner), as well as [ItemRequestProxy](https://wiki.factorio.com/Prototype/ItemRequestProxy) and [EntityGhost](https://wiki.factorio.com/Prototype/EntityGhost) are assigned a unit number. Returns `nil` otherwise.
--- @field ghost_unit_number uint? The [unit_number](LuaEntity::unit_number) of the entity contained in this ghost. It is the same as the unit number of the [EntityWithOwner](https://wiki.factorio.com/Prototype/EntityWithOwner) that was destroyed to create this ghost. If it was created by other means, or if the inner entity does not support unit numbers, this property is `nil`.
--- @field mining_progress double? The mining progress for this mining drill. Is a number in range [0, mining_target.prototype.mineable_properties.mining_time]. `nil` if this isn't a mining drill.
--- @field bonus_mining_progress double? The bonus mining progress for this mining drill. Read yields a number in range [0, mining_target.prototype.mineable_properties.mining_time]. `nil` if this isn't a mining drill.
--- @field power_production double The power production specific to the ElectricEnergyInterface entity type.
--- @field power_usage double The power usage specific to the ElectricEnergyInterface entity type.
--- @field bounding_box BoundingBox [LuaEntityPrototype::collision_box](LuaEntityPrototype::collision_box) around entity's given position and respecting the current entity orientation.
--- @field secondary_bounding_box BoundingBox? The secondary bounding box of this entity or `nil` if it doesn't have one. This only exists for curved rails, and is automatically determined by the game.
--- @field selection_box BoundingBox [LuaEntityPrototype::selection_box](LuaEntityPrototype::selection_box) around entity's given position and respecting the current entity orientation.
--- @field secondary_selection_box BoundingBox? The secondary selection box of this entity or `nil` if it doesn't have one. This only exists for curved rails, and is automatically determined by the game.
--- @field mining_target LuaEntity? The mining target, if any.
--- @field circuit_connected_entities { green: LuaEntity[], red: LuaEntity[] }? Entities that are directly connected to this entity via the circuit network. `nil` if this entity can't be connected to the circuit network.
--- @field circuit_connection_definitions CircuitConnectionDefinition[]? The connection definition for entities that are directly connected to this entity via the circuit network. `nil` if this entity can't be connected to the circuit network.
--- @field request_slot_count uint The index of the configured request with the highest index for this entity. This means 0 if no requests are set and e.g. 20 if the 20th request slot is configured.
--- @field filter_slot_count uint The number of filter slots this inserter, loader, or logistic storage container has. 0 if not one of those entities.
--- @field loader_container LuaEntity? The container entity this loader is pointing at/pulling from depending on the [LuaEntity::loader_type](LuaEntity::loader_type), if any.
--- @field grid LuaEquipmentGrid? This entity's equipment grid, if any.
--- @field graphics_variation uint8? The graphics variation for this entity. `nil` if this entity doesn't use graphics variations.
--- @field tree_color_index uint8 Index of the tree color.
--- @field tree_color_index_max uint8 Maximum index of the tree colors.
--- @field tree_stage_index uint8 Index of the tree stage.
--- @field tree_stage_index_max uint8 Maximum index of the tree stages.
--- @field tree_gray_stage_index uint8 Index of the tree gray stage
--- @field tree_gray_stage_index_max uint8 Maximum index of the tree gray stages.
--- @field burner LuaBurner? The burner energy source for this entity, if any.
--- @field shooting_target LuaEntity? The shooting target for this turret, if any. Can't be set to `nil` via script.
--- @field proxy_target LuaEntity? The target entity for this item-request-proxy, if any.
--- @field stickers LuaEntity[]? The sticker entities attached to this entity, if any.
--- @field sticked_to LuaEntity The entity this sticker is sticked to.
--- @field parameters ProgrammableSpeakerParameters 
--- @field alert_parameters ProgrammableSpeakerAlertParameters 
--- @field electric_network_statistics LuaFlowStatistics The electric network statistics for this electric pole.
--- @field inserter_target_pickup_count uint Returns the current target pickup count of the inserter.
--- @field inserter_stack_size_override uint Sets the stack size limit on this inserter. If the stack size is > than the force stack size limit the value is ignored.
--- @field products_finished uint The number of products this machine finished crafting in its lifetime.
--- @field spawner LuaEntity? The spawner associated with this unit entity, if any.
--- @field units LuaEntity[] The units associated with this spawner entity.
--- @field power_switch_state boolean The state of this power switch.
--- @field effects ModuleEffects? The effects being applied to this entity, if any. For beacons, this is the effect the beacon is broadcasting.
--- @field beacons_count uint? Number of beacons affecting this effect receiver. Can only be used when the entity has an effect receiver (AssemblingMachine, Furnace, Lab, MiningDrills)
--- @field infinity_container_filters InfinityInventoryFilter[] The filters for this infinity container.
--- @field remove_unfiltered_items boolean Whether items not included in this infinity container filters should be removed from the container.
--- @field character_corpse_player_index uint The player index associated with this character corpse.
--- @field character_corpse_tick_of_death uint The tick this character corpse died at.
--- @field character_corpse_death_cause LocalisedString The reason this character corpse character died. `""` if there is no reason.
--- @field associated_player PlayerIdentification|LuaPlayer? The player this character is associated with, if any. Set to `nil` to clear.  The player will be automatically disassociated when a controller is set on the character. Also, all characters associated to a player will be logged off when the player logs off in multiplayer.  Reading this property will return a [LuaPlayer](LuaPlayer), while [PlayerIdentification](PlayerIdentification) can be used when writing.
--- @field tick_of_last_attack uint The last tick this character entity was attacked.
--- @field tick_of_last_damage uint The last tick this character entity was damaged.
--- @field splitter_filter LuaItemPrototype? The filter for this splitter, if any is set.
--- @field inserter_filter_mode string? The filter mode for this filter inserter. Either `"whitelist"` or `"blacklist"`. `nil` if this inserter doesn't use filters.
--- @field splitter_input_priority string The input priority for this splitter. Either `"left"`, `"none"`, or `"right"`.
--- @field splitter_output_priority string The output priority for this splitter. Either `"left"`, `"none"`, or `"right"`.
--- @field armed boolean Whether this land mine is armed.
--- @field recipe_locked boolean When locked; the recipe in this assembling machine can't be changed by the player.
--- @field connected_rail LuaEntity? The rail entity this train stop is connected to, if any.
--- @field connected_rail_direction defines.rail_direction Rail direction to which this train stop is binding. This returns a value even when no rails are present.
--- @field trains_in_block uint The number of trains in this rail block for this rail entity.
--- @field timeout uint The timeout that's left on this landmine in ticks. It describes the time between the landmine being placed and it being armed.
--- @field neighbour_bonus double The current total neighbour bonus of this reactor.
--- @field ai_settings LuaAISettings The ai settings of this unit.
--- @field highlight_box_type string The hightlight box type of this highlight box entity.
--- @field highlight_box_blink_interval uint The blink interval of this highlight box entity. 0 indicates no blink.
--- @field status defines.entity_status? The status of this entity, if any.
--- @field enable_logistics_while_moving boolean Whether equipment grid logistics are enabled while this vehicle is moving.
--- @field render_player PlayerIdentification|LuaPlayer? The player that this `simple-entity-with-owner`, `simple-entity-with-force`, `flying-text`, or `highlight-box` is visible to. `nil` when this entity is rendered for all players.  Reading this property will return a [LuaPlayer](LuaPlayer), while [PlayerIdentification](PlayerIdentification) can be used when writing.
--- @field render_to_forces ForceIdentification[]? The forces that this `simple-entity-with-owner`, `simple-entity-with-force`, or `flying-text` is visible to. `nil` or an empty array when this entity is rendered for all forces.
--- @field pump_rail_target LuaEntity? The rail target of this pump, if any.
--- @field moving boolean Returns true if this unit is moving.
--- @field electric_network_id uint? Returns the id of the electric network that this entity is connected to, if any.
--- @field allow_dispatching_robots boolean Whether this character's personal roboports are allowed to dispatch robots.
--- @field auto_launch boolean Whether this rocket silo automatically launches the rocket when cargo is inserted.
--- @field energy_generated_last_tick double How much energy this generator generated in the last tick.
--- @field storage_filter LuaItemPrototype The storage filter for this logistic storage container.
--- @field request_from_buffers boolean Whether this requester chest is set to also request from buffer chests.
--- @field corpse_expires boolean Whether this corpse will ever fade away.
--- @field corpse_immune_to_entity_placement boolean If true, corpse won't be destroyed when entities are placed over it. If false, whether corpse will be removed or not depends on value of CorpsePrototype::remove_on_entity_placement.
--- @field tags Tags? The tags associated with this entity ghost. `nil` if this is not an entity ghost.
--- @field command Command? The command given to this unit, if any.
--- @field distraction_command Command? The distraction command given to this unit, if any.
--- @field time_to_next_effect uint The ticks until the next trigger effect of this smoke-with-trigger.
--- @field autopilot_destination MapPosition? Destination of this spidertron's autopilot, if any. Writing `nil` clears all destinations.
--- @field autopilot_destinations MapPosition[] The queued destination positions of spidertron's autopilot.
--- @field trains_count uint Amount of trains related to this particular train stop. Includes train stopped at this train stop (until it finds a path to next target) and trains having this train stop as goal or waypoint.
--- @field trains_limit uint Amount of trains above which no new trains will be sent to this train stop. Writing nil will disable the limit (will set a maximum possible value).
--- @field is_entity_with_force boolean (deprecated by 1.1.51) If this entity is a MilitaryTarget. Returns same value as LuaEntity::is_military_target
--- @field is_military_target boolean Whether this entity is a MilitaryTarget. Can be written to if [LuaEntityPrototype::allow_run_time_change_of_is_military_target](LuaEntityPrototype::allow_run_time_change_of_is_military_target) returns `true`.
--- @field is_entity_with_owner boolean If this entity is EntityWithOwner
--- @field is_entity_with_health boolean If this entity is EntityWithHealth
--- @field combat_robot_owner LuaEntity? The owner of this combat robot, if any.
--- @field link_id uint The link ID this linked container is using.
--- @field follow_target LuaEntity? The follow target of this spidertron, if any.
--- @field follow_offset Vector? The follow offset of this spidertron, if any entity is being followed. This is randomized each time the follow entity is set.
--- @field linked_belt_type string Type of linked belt: it is either `"input"` or `"output"`. Changing type will also flip direction so the belt is out of the same side
--- @field linked_belt_neighbour LuaEntity? Neighbour to which this linked belt is connected to, if any.
--- @field radar_scan_progress float The current radar scan progress, as a number in range [0, 1].
--- @field rocket_silo_status defines.rocket_silo_status The status of this rocket silo entity.
--- @field tile_width uint Specifies the tiling size of the entity, is used to decide, if the center should be in the center of the tile (odd tile size dimension) or on the tile border (even tile size dimension). Uses the current direction of the entity.
--- @field tile_height uint Specifies the tiling size of the entity, is used to decide, if the center should be in the center of the tile (odd tile size dimension) or on the tile border (even tile size dimension). Uses the current direction of the entity.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaEntity = { }

----
--- Adds the given position to this spidertron's autopilot's queue of destinations.
--- 
--- @param position MapPosition The position the spidertron should move to.
----
function LuaEntity.add_autopilot_destination(position) end

----
--- Offer a thing on the market.
--- 
--- @param offer Offer 
----
function LuaEntity.add_market_item(offer) end

----
--- Checks if the entity can be destroyed
--- 
--- @return boolean Whether the entity can be destroyed.
----
function LuaEntity.can_be_destroyed() end

----
--- Whether this character can shoot the given entity or position.
--- 
--- @param target LuaEntity 
--- @param position MapPosition 
--- @return boolean 
----
function LuaEntity.can_shoot(target, position) end

----
--- Can wires reach between these entities.
--- 
--- @param entity LuaEntity 
--- @return boolean 
----
function LuaEntity.can_wires_reach(entity) end

----
--- Cancels deconstruction if it is scheduled, does nothing otherwise.
--- 
--- @param force ForceIdentification The force who did the deconstruction order.
--- @param player PlayerIdentification? The player to set the `last_user` to if any.
----
function LuaEntity.cancel_deconstruction(force, player) end

----
--- Cancels upgrade if it is scheduled, does nothing otherwise.
--- 
--- @param force ForceIdentification The force who did the upgrade order.
--- @param player PlayerIdentification? The player to set the last_user to if any.
--- @return boolean Whether the cancel was successful.
----
function LuaEntity.cancel_upgrade(force, player) end

----
--- Remove all fluids from this entity.
--- 
----
function LuaEntity.clear_fluid_inside() end

----
--- Removes all offers from a market.
--- 
----
function LuaEntity.clear_market_items() end

----
--- Clear a logistic requester slot.
--- 
--- @param slot uint The slot index.
----
function LuaEntity.clear_request_slot(slot) end

----
--- @class LuaEntity.clone_parameters
--- Parameters class for the method `LuaEntity.clone`
--- 
--- @field position MapPosition The destination position
--- @field surface LuaSurface? The destination surface
--- @field force ForceIdentification? 
--- @field create_build_effect_smoke boolean? If false, the building effect smoke will not be shown around the new entity.
----

----
--- Clones this entity.
--- 
--- @param args LuaEntity.clone_parameters Table containing arguments for the method.
--- @see LuaEntity.clone_parameters
--- @return LuaEntity? The cloned entity or `nil` if this entity can't be cloned/can't be cloned to the given location.
----
function LuaEntity.clone(args) end

----
--- Connects current linked belt with another one.
--- 
--- Neighbours have to be of different type. If given linked belt is connected to something else it will be disconnected first. If provided neighbour is connected to something else it will also be disconnected first. Automatically updates neighbour to be connected back to this one.
--- 
--- @param neighbour LuaEntity Another linked belt or entity ghost containing linked belt to connect or nil to disconnect
----
function LuaEntity.connect_linked_belts(neighbour) end

----
--- Connect two devices with a circuit wire or copper cable. Depending on which type of connection should be made, there are different procedures:
--- 
--- - To connect two electric poles, `target` must be a [LuaEntity](LuaEntity) that specifies another electric pole. This will connect them with copper cable.
--- - To connect two devices with circuit wire, `target` must be a table of type [WireConnectionDefinition](WireConnectionDefinition).
--- 
--- @param target WireConnectionDefinition|LuaEntity The target with which to establish a connection.
--- @return boolean Whether the connection was successfully formed.
----
function LuaEntity.connect_neighbour(target) end

----
--- Connects the rolling stock in the given direction.
--- 
--- @param direction defines.rail_direction 
--- @return boolean Whether any connection was made
----
function LuaEntity.connect_rolling_stock(direction) end

----
--- Copies settings from the given entity onto this entity.
--- 
--- @param entity LuaEntity 
--- @param by_player PlayerIdentification? If provided, the copying is done 'as' this player and [on_entity_settings_pasted](on_entity_settings_pasted) is triggered.
--- @return table<string,uint> Any items removed from this entity as a result of copying the settings.
----
function LuaEntity.copy_settings(entity, by_player) end

----
--- Creates the same smoke that is created when you place a building by hand. You can play the building sound to go with it by using [LuaSurface::play_sound](LuaSurface::play_sound), eg: entity.surface.play_sound{path="entity-build/"..entity.prototype.name, position=entity.position}
--- 
----
function LuaEntity.create_build_effect_smoke() end

----
--- Damages the entity.
--- 
--- @param damage float The amount of damage to be done.
--- @param force ForceIdentification The force that will be doing the damage.
--- @param type string? The type of damage to be done, defaults to "impact". Can't be `nil`.
--- @param dealer LuaEntity? The entity to consider as the damage dealer. Needs to be on the same surface as the entity being damaged.
--- @return float the total damage actually applied after resistances.
----
function LuaEntity.damage(damage, force, type, dealer) end

----
--- Depletes and destroys this resource entity.
--- 
----
function LuaEntity.deplete() end

----
--- @class LuaEntity.destroy_parameters
--- Parameters class for the method `LuaEntity.destroy`
--- 
--- @field do_cliff_correction boolean? Whether neighbouring cliffs should be corrected. Defaults to `false`.
--- @field raise_destroy boolean? If `true`, [script_raised_destroy](script_raised_destroy) will be called. Defaults to `false`.
----

----
--- Destroys the entity.
--- 
--- @param args LuaEntity.destroy_parameters? Table containing arguments for the method.
--- @see LuaEntity.destroy_parameters
--- @return boolean Returns `false` if the entity was valid and destruction failed, `true` in all other cases.
----
function LuaEntity.destroy(args) end

----
--- Immediately kills the entity. Does nothing if the entity doesn't have health.
--- 
--- Unlike [LuaEntity::destroy](LuaEntity::destroy), `die` will trigger the [on_entity_died](on_entity_died) event and the entity will produce a corpse and drop loot if it has any.
--- 
--- @param force ForceIdentification? The force to attribute the kill to.
--- @param cause LuaEntity? The cause to attribute the kill to.
--- @return boolean Whether the entity was successfully killed.
----
function LuaEntity.die(force, cause) end

----
--- Disconnects linked belt from its neighbour.
--- 
----
function LuaEntity.disconnect_linked_belts() end

----
--- Disconnect circuit wires or copper cables between devices. Depending on which type of connection should be cut, there are different procedures:
--- 
--- - To remove all copper cables, leave the `target` parameter blank: `pole.disconnect_neighbour()`.
--- - To remove all wires of a specific color, set `target` to [defines.wire_type.red](defines.wire_type.red) or [defines.wire_type.green](defines.wire_type.green).
--- - To remove a specific copper cable between two electric poles, `target` must be a [LuaEntity](LuaEntity) that specifies the other pole: `pole1.disconnect_neighbour(pole2)`.
--- - To remove a specific circuit wire, `target` must be a table of type [WireConnectionDefinition](WireConnectionDefinition).
--- 
--- @param target WireConnectionDefinition|LuaEntity|defines.wire_type? The target with which to cut a connection.
----
function LuaEntity.disconnect_neighbour(target) end

----
--- Tries to disconnect this rolling stock in the given direction.
--- 
--- @param direction defines.rail_direction 
--- @return boolean If anything was disconnected
----
function LuaEntity.disconnect_rolling_stock(direction) end

----
--- Returns a table with all entities affected by this beacon
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_beacon_effect_receivers() end

----
--- Returns a table with all beacons affecting this effect receiver. Can only be used when the entity has an effect receiver (AssemblingMachine, Furnace, Lab, MiningDrills)
--- 
--- @return LuaEntity[]? 
----
function LuaEntity.get_beacons() end

----
--- Get the source of this beam.
--- 
--- @return BeamTarget? 
----
function LuaEntity.get_beam_source() end

----
--- Get the target of this beam.
--- 
--- @return BeamTarget? 
----
function LuaEntity.get_beam_target() end

----
--- The burnt result inventory for this entity or `nil` if this entity doesn't have a burnt result inventory.
--- 
--- @return LuaInventory? 
----
function LuaEntity.get_burnt_result_inventory() end

----
--- Returns all child signals. Child signals can be either RailSignal or RailChainSignal. Child signals are signals which are checked by this signal to determine a chain state.
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_child_signals() end

----
--- 
--- @param wire defines.wire_type Wire color of the network connected to this entity.
--- @param circuit_connector defines.circuit_connector_id? The connector to get circuit network for. Must be specified for entities with more than one circuit network connector.
--- @return LuaCircuitNetwork? The circuit network or nil.
----
function LuaEntity.get_circuit_network(wire, circuit_connector) end

----
--- @class LuaEntity.get_connected_rail_parameters
--- Parameters class for the method `LuaEntity.get_connected_rail`
--- 
--- @field rail_direction defines.rail_direction 
--- @field rail_connection_direction defines.rail_connection_direction 
----

----
--- 
--- @param args LuaEntity.get_connected_rail_parameters Table containing arguments for the method.
--- @see LuaEntity.get_connected_rail_parameters
--- @return LuaEntity? Rail connected in the specified manner to this one, `nil` if unsuccessful.
--- @return defines.rail_direction? Rail direction of the returned rail which points to origin rail
--- @return defines.rail_connection_direction? Turn to be taken when going back from returned rail to origin rail
----
function LuaEntity.get_connected_rail(args) end

----
--- Get the rails that this signal is connected to.
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_connected_rails() end

----
--- Gets rolling stock connected to the given end of this stock.
--- 
--- @param direction defines.rail_direction 
--- @return LuaEntity? The rolling stock connected at the given end, `nil` if none is connected there.
--- @return defines.rail_direction? The rail direction of the connected rolling stock if any.
----
function LuaEntity.get_connected_rolling_stock(direction) end

----
--- Gets the control behavior of the entity (if any).
--- 
--- @return LuaControlBehavior? The control behavior or `nil`.
----
function LuaEntity.get_control_behavior() end

----
--- Returns the amount of damage to be taken by this entity.
--- 
--- @return float? `nil` if this entity does not have health.
----
function LuaEntity.get_damage_to_be_taken() end

----
--- Gets the driver of this vehicle if any.
--- 
--- @return LuaPlayer|LuaEntity? `nil` if the vehicle contains no driver. To check if there's a passenger see [LuaEntity::get_passenger](LuaEntity::get_passenger).
----
function LuaEntity.get_driver() end

----
--- Get the filter for a slot in an inserter, loader, or logistic storage container.
--- 
--- @param slot_index uint Index of the slot to get the filter for.
--- @return string? Prototype name of the item being filtered. `nil` if the given slot has no filter.
----
function LuaEntity.get_filter(slot_index) end

----
--- Get amounts of all fluids in this entity.
--- 
--- @return table<string,double> The amounts, indexed by fluid names.
----
function LuaEntity.get_fluid_contents() end

----
--- Get the amount of all or some fluid in this entity.
--- 
--- @param fluid string? Prototype name of the fluid to count. If not specified, count all fluids.
--- @return double 
----
function LuaEntity.get_fluid_count(fluid) end

----
--- The fuel inventory for this entity or `nil` if this entity doesn't have a fuel inventory.
--- 
--- @return LuaInventory? 
----
function LuaEntity.get_fuel_inventory() end

----
--- The health ratio of this entity between 1 and 0 (for full health and no health respectively).
--- 
--- @return float? `nil` if this entity doesn't have health.
----
function LuaEntity.get_health_ratio() end

----
--- Gets the heat setting for this heat interface.
--- 
--- @return HeatSetting 
----
function LuaEntity.get_heat_setting() end

----
--- Returns all signals guarding entrance to a rail block this rail belongs to.
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_inbound_signals() end

----
--- Gets the filter for this infinity container at the given index, or `nil` if the filter index doesn't exist or is empty.
--- 
--- @param index uint The index to get.
--- @return InfinityInventoryFilter? 
----
function LuaEntity.get_infinity_container_filter(index) end

----
--- Gets the filter for this infinity pipe, or `nil` if the filter is empty.
--- 
--- @return InfinityPipeFilter? 
----
function LuaEntity.get_infinity_pipe_filter() end

----
--- Gets all the `LuaLogisticPoint`s that this entity owns. Optionally returns only the point specified by the index parameter.
--- 
--- @param index defines.logistic_member_index? If provided, only returns the `LuaLogisticPoint` specified by this index.
--- @return LuaLogisticPoint[]|LuaLogisticPoint? 
----
function LuaEntity.get_logistic_point(index) end

----
--- Get all offers in a market as an array.
--- 
--- @return Offer[] 
----
function LuaEntity.get_market_items() end

----
--- Get the maximum transport line index of a belt or belt connectable entity.
--- 
--- @return uint 
----
function LuaEntity.get_max_transport_line_index() end

----
--- Read a single signal from the combined circuit networks.
--- 
--- @param signal SignalID The signal to read.
--- @param circuit_connector defines.circuit_connector_id? The connector to get signals for. Must be specified for entities with more than one circuit network connector.
--- @return int The current value of the signal.
----
function LuaEntity.get_merged_signal(signal, circuit_connector) end

----
--- The merged circuit network signals or `nil` if there are no signals.
--- 
--- @param circuit_connector defines.circuit_connector_id? The connector to get signals for. Must be specified for entities with more than one circuit network connector.
--- @return Signal[]? The sum of signals on both the red and green networks, or `nil` if it doesn't have a circuit connector.
----
function LuaEntity.get_merged_signals(circuit_connector) end

----
--- Inventory for storing modules of this entity; `nil` if this entity has no module inventory.
--- 
--- @return LuaInventory? 
----
function LuaEntity.get_module_inventory() end

----
--- Gets (and or creates if needed) the control behavior of the entity.
--- 
--- @return LuaControlBehavior? The control behavior or `nil`.
----
function LuaEntity.get_or_create_control_behavior() end

----
--- Returns all signals guarding exit from a rail block this rail belongs to.
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_outbound_signals() end

----
--- Gets the entity's output inventory if it has one.
--- 
--- @return LuaInventory? A reference to the entity's output inventory.
----
function LuaEntity.get_output_inventory() end

----
--- Returns all parent signals. Parent signals are always RailChainSignal. Parent signals are those signals that are checking state of this signal to determine their own chain state.
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_parent_signals() end

----
--- Gets the passenger of this car or spidertron if any.
--- 
--- @return LuaPlayer|LuaEntity? `nil` if the vehicle contains no passenger. To check if there's a driver see [LuaEntity::get_driver](LuaEntity::get_driver).
----
function LuaEntity.get_passenger() end

----
--- The radius of this entity.
--- 
--- @return double 
----
function LuaEntity.get_radius() end

----
--- Get the rail at the end of the rail segment this rail is in.
--- 
--- @param direction defines.rail_direction 
--- @return LuaEntity The rail entity.
--- @return defines.rail_direction A rail direction pointing out of the rail segment from the end rail.
----
function LuaEntity.get_rail_segment_end(direction) end

